def version(name: str, url: str) -> str: ...
def info(name: str, url: str) -> dict: ...
def main(): ...

class stats:
    def __init__(self, name: str, url: str): ...
    def get_total() -> dict: ...
    def dataset() -> dict: ...


import requests
import os.path as path
import subprocess as sp
import wget 

from sys import platform

# directory = path.dirname(__file__)

filename = 'temp.tmp'

windows = ['win32', 'windows']

info_data = {
    'name': None,
	'author': None,
	'license': None,
	'summary': None,
	'previous_version': '0.0.0',
	'latest_version': '0.0.0',
    'downloads': 0
}

dataset = {
    'with_mirrors': None,
    'without_mirrors': None,
}

def clean():
    if platform not in windows:
            sp.getoutput(f'rm {filename}')
    else:
        sp.getoutput(f'del {filename}')

def get_total():
    # sp.getoutput(f'wget -O {filename} ')

    file = requests.get(f"https://static.pepy.tech/personalized-badge/{info_data['name']}?period=total&units=none").content.decode()
    file = file.split('downloads/month</text>')
    portion = file[-1]
    portion = portion.replace(portion[portion.index('<text'):portion.index('>') + 1], '')
    total = portion.replace(portion[portion.index('</text>'):], '')

    return int(total)

def get_previous(name):
    rss = requests.get(f'https://pypi.org/rss/project/{name}/releases.xml').content.decode()

    rss = rss[rss.index('<item>'):].replace(rss[rss.index('</channel>'):], '')
    rss = rss.replace('<item>', '').split('</item>')

    for idx in range(len(rss)):
        rss[idx] = rss[idx].replace('\n', '').split()
    else:
        previous = rss[1]
        return previous[0].replace('<title>', '').replace('</title>', '').strip()
    
def get_info():
    wget.download(f'https://pypistats.org/packages/{info_data["name"]}', filename, None)
    with open(filename, 'r', errors='ignore') as file:
        file = file.read()
        file = file.replace(file[:file.index(f'<h1>{info_data["name"]}</h1>')], '')
        portion  = file.replace(file[file.index('<script>'):], '')
        details = portion[portion.index('Author:'):]

        trim = details[details.index("Latest version:"):]
        
        details = details.replace(trim[trim.index('<br>'):], '').split('<br>')

        for detail in details:
            identity, value = detail.split(':')
            value = value.strip()

            identity = identity.strip().split()[-1].lower()
            if identity == 'version':
                info_data['latest_version'] = value
            else:
                info_data[identity] = value
        else:
            dataset['with_mirrors'] = with_mirrors(file)
            info_data['downloads'] = f'{get_total():,}'
    
    info_data['previous_version'] = get_previous(info_data['name'])
    clean()



def package_name(name, url):
    name = name.lower()

    def default():
        if name != None:
            info_data['name'] = name
        
    if url != None:
        wget.download(url, filename, None)

        with open(filename, 'r', errors='ignore') as file:
            lines = file.readlines()
            start = '<title>'
            end = '</title>'
            for line in lines:
                if start in line and end in line:
                    line = line.strip().removeprefix(start).removesuffix(end)
                    info_data['name'] = line.split(' · ')[0]
                    break
            else:
                default()
    else:
        default()

def get_data(text):
    data = text[text.index('"x":'):]
    data = data[:data.index('}')]

    return convert_str(data)


def convert_str(data):
    x = data[:data.index('],') + 1]
    y = data[data.index('],') + 2: ]
    
    dates = x.split(':')
    downloads = y.split(":")

    data = {}
    data['dates'] = dates[-1]
    data['downloads'] = downloads[-1]

    keys = ['dates', 'downloads']

    for key in keys:
        symbols = ['[', ']', '"']
        data_val = data[key]
        
        for symbol in symbols:
            data_val = data_val.replace(symbol, '')
        else:
            data[key] = [val.strip() for val in data_val.split(',')]			
    else:
        data['downloads'] = [int(download) for download in data['downloads']]
        return data


def with_mirrors(read_data):
    data = get_data(read_data)
    return data

def without_mirrors(read_data):
    data = get_data(read_data[read_data.index('Without_Mirrors'):])
    return data


def scan_stats(name: str = None, url: str = None, mode: str = 'a'):
    '''
    mode representation
    a - all :with_mirrors
    s - some :without_mirrors
    '''
    
    package_name(name, url)

    wget.download(f'https://pypistats.org/packages/{info_data["name"]}', filename, None)
        
    with open(filename, 'r', errors='ignore') as file:
        file = file.read()

        clean()

        if mode == 'a':
            return with_mirrors(file)
        elif mode == 's':
            return without_mirrors(file)
    


import subprocess as sp

from .handler import *

__all__ = ['version', 'info', 'stats', 'main']

clean() # clean temp file not cleaned to prevent error.

def version(name: str = None, url: str = None):
    '''
        Note: please use "info" instead to ensure result accuracy.
        only use this if your preference is speed.
    '''
    try:
        if name != None:
            url = f'https://pypi.org/project/{name}'
            
        module_name = url.split('/')

        module_name = [name for name in module_name if len(name) != 0][-1]

        wget.download(url, filename, None)

        with open(filename, 'r', errors='ignore') as file:
            file = file.readlines()
            
            line = [line for line in file if f"{module_name} " in line and "title" not in line]
            line = line[0].strip()
            version = line.split()[-1]

        clean()
        return module_name, version
    except:
        ...


class stats:
    def __init__(self, name: str = None, url: str = None):
        dataset['with_mirrors'] = scan_stats(name, url, mode='a')
        clean()
        dataset['without_mirrors'] = scan_stats(name, url, mode='s')
        clean()
        
    def get_total(self):
        return get_total()
    
    def dataset(self):
        return dataset


def info(name: str = None, url: str = None):
    try:
        package_name(name, url)
        get_info()
    except:
        ...

    return info_data


def make_table(date,details):
    name = []

    last_day = []
    last_seven_days = []
    last_thirty_days = []

    total = []

    marker = []

    def make_table_value(details):
        max = 0
        for detail in details:
            if len(detail) > max:
                max = len(detail)

        marker.append(f'{"—" * (max + 2)}+')

        def data(val):
            return f'{(" " * ((max + 1) - len(val))) + val} |'

        def handle_name(val):
            remainder = max - len(val)
            space_value = int(remainder / 2)
            space_value = " " * space_value
            value = f'{space_value}{val}{space_value}'

            return f' {" " * (max - len(value))}{value} |'


        name.append(handle_name(details[0]))

        last_day.append(data(details[1]))
        last_seven_days.append(data(details[2]))
        last_thirty_days.append(data(details[3]))

        total.append(data(details[4]))

    for detail in details:
        make_table_value(detail)

    table = f'''
+——————————————+{''.join(marker)}
|    Name(s)   |{''.join(name)}
+——————————————+{''.join(marker)}
|  {date[0]}  |{''.join(last_day)}
| Last 07 Days |{''.join(last_seven_days)}
| Last 30 Days |{''.join(last_thirty_days)}
+——————————————+{''.join(marker)}
|   Total(s)   |{''.join(total)}
+——————————————+{''.join(marker)}'''

    return table

def main():
    try:
        package_name = input('Package Name(s): ')
        
        package_names = package_name.split(',')
        
        date = ['']

        def get_stats(name):
            name = name.strip()

            statistics = stats(name)
            set = statistics.dataset()
            
            set_one = set['with_mirrors']
            # set_two = set['without_mirrors']

            dates = set_one['dates']
            date[0] = [dates[-1]]

            downloads = set_one['downloads']

            day = f"{downloads[-1]:,}"
        
            week = 0
            for i in downloads[-7:]:
                week += i
            else:
                week = f"{week:,}"

            month = 0
            for i in downloads[-30:]:
                month += i
            else:
                month = f"{month:,}"

            return [name, day, week, month, f'{statistics.get_total():,}']
        
        # data = []

        # for i in trange(len(package_names)):
        #     data.append(get_stats(package_names[i]))

        data = [get_stats(name) for name in package_names] 
        print(make_table(date[0], data))
    except:
        print('No Result...')
        
if __name__ == '__main__':
    main()


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as readme:
    LONG_DESCRIPTION = readme.read()


VERSION = '0.0.8'
DESCRIPTION = 'Get PyPi Package information.'
KEYWORDS = ['pypi', 'information', 'package', 'module', 'stats', 'info', 'scrape']


CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Environment :: Console',
    'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
]

def init_ref(folder):
    directory = [folder]
    data_ref = dict()
    excluded = ['docs', 'files']

    for dir in directory:
        files = []
        for item in os.scandir(dir):
            if item.is_dir():
                directory.append(item.path)
            elif item.is_file():
                name = item.name
                if name.endswith('.py') or name.endswith('.pyi'):
                    files.append(name)
        else:
            if len(files) != 0:
                for item in excluded:
                    if dir.endswith(item):
                        break
                else:
                    data_ref[dir] = files
    else:
        return data_ref

data_ref = init_ref('scrapypi')

setup(
    name="scrapypi",
    version=VERSION,
    author="Erasmus A. Junior",
    author_email="eirasmx@pm.me",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    licence='GNU LGPLv3',
    long_description=LONG_DESCRIPTION,
    packages=[data for data in data_ref],
    package_data=data_ref,
    python_requires='>=3.7',
    install_requires=['wget', 'requests'],
    keywords=KEYWORDS,
    classifiers=CLASSIFIERS,
    py_modules=['scrapypi'],
    entry_points = {'console_scripts':['scrapypi = scrapypi:main']},
)
